@@ -12,8 +12,7 @@
 use std::borrow::Cow;
 
 use crate::{
-    decoders::encoded_word::parse_encoded_word, parsers::message_stream::MessageStream, Addr,
-    Address, Group,
+    decoders::encoded_word::decode_rfc2047, parsers::message::MessageStream, Addr, Address, Group,
 };
 
 impl<'x> Default for Address<'x> {
@@ -40,7 +39,6 @@ pub struct AddressParser<'x> {
     token_start: usize,
     token_end: usize,
 
-    is_token_safe: bool,
     is_token_email: bool,
     is_token_start: bool,
     is_escaped: bool,
@@ -64,13 +62,7 @@ pub fn add_token<'x>(
     add_trail_space: bool,
 ) {
     if parser.token_start > 0 {
-        let token = stream
-            .get_string(
-                parser.token_start - 1,
-                parser.token_end,
-                parser.is_token_safe,
-            )
-            .unwrap();
+        let token = String::from_utf8_lossy(&stream.data[parser.token_start - 1..parser.token_end]);
         let mut add_space = false;
         let list = match parser.state {
             AddressState::Address => &mut parser.mail_tokens,
@@ -100,7 +92,6 @@ pub fn add_token<'x>(
         }
 
         parser.token_start = 0;
-        parser.is_token_safe = true;
         parser.is_token_email = false;
         parser.is_token_start = true;
         parser.is_escaped = false;
@@ -233,12 +224,11 @@ pub fn add_group(parser: &mut AddressParser) {
         });
 }
 
-pub fn parse_address<'x>(stream: &MessageStream<'x>) -> Address<'x> {
+pub fn parse_address<'x>(stream: &mut MessageStream<'x>) -> Address<'x> {
     let mut parser = AddressParser {
         token_start: 0,
         token_end: 0,
 
-        is_token_safe: true,
         is_token_email: false,
         is_token_start: true,
         is_escaped: false,
@@ -256,13 +246,18 @@ pub fn parse_address<'x>(stream: &MessageStream<'x>) -> Address<'x> {
         result: Vec::new(),
     };
 
-    while let Some(ch) = stream.next() {
+    let mut iter = stream.data[stream.pos..].iter();
+
+    while let Some(ch) = iter.next() {
+        stream.pos += 1;
+
         match ch {
             b'\n' => {
                 add_token(&mut parser, stream, false);
-                match stream.peek() {
+                match stream.data.get(stream.pos) {
                     Some(b' ' | b'\t') => {
-                        stream.advance(1);
+                        iter.next();
+                        stream.pos += 1;
                         if !parser.is_token_start {
                             parser.is_token_start = true;
                         }
@@ -274,7 +269,7 @@ pub fn parse_address<'x>(stream: &MessageStream<'x>) -> Address<'x> {
             b'\\' if parser.state != AddressState::Name => {
                 if parser.token_start > 0 {
                     if parser.state == AddressState::Quote {
-                        parser.token_end = stream.get_pos() - 1;
+                        parser.token_end = stream.pos - 1;
                     }
                     add_token(&mut parser, stream, false);
                 }
@@ -315,7 +310,7 @@ pub fn parse_address<'x>(stream: &MessageStream<'x>) -> Address<'x> {
                 parser.is_token_email = true;
             }
             b'=' if parser.is_token_start && !parser.is_escaped => {
-                if let Some(token) = parse_encoded_word(stream) {
+                if let (d_bytes_read, Some(token)) = decode_rfc2047(stream, stream.pos) {
                     let add_space = parser.state != AddressState::Quote; // Make borrow-checker happy
                     add_token(&mut parser, stream, add_space);
                     (if parser.state != AddressState::Comment {
@@ -323,7 +318,9 @@ pub fn parse_address<'x>(stream: &MessageStream<'x>) -> Address<'x> {
                     } else {
                         &mut parser.comment_tokens
                     })
-                    .push(token);
+                    .push(token.into());
+                    stream.pos += d_bytes_read;
+                    iter = stream.data[stream.pos..].iter();
                     continue;
                 }
             }
@@ -336,10 +333,10 @@ pub fn parse_address<'x>(stream: &MessageStream<'x>) -> Address<'x> {
                 }
                 if parser.state == AddressState::Quote {
                     if parser.token_start == 0 {
-                        parser.token_start = stream.get_pos();
+                        parser.token_start = stream.pos;
                         parser.token_end = parser.token_start;
                     } else {
-                        parser.token_end = stream.get_pos();
+                        parser.token_end = stream.pos;
                     }
                 }
                 continue;
@@ -373,12 +370,7 @@ pub fn parse_address<'x>(stream: &MessageStream<'x>) -> Address<'x> {
                 add_group(&mut parser);
                 continue;
             }
-            0..=0x7f => (),
-            _ => {
-                if parser.is_token_safe {
-                    parser.is_token_safe = false;
-                }
-            }
+            _ => (),
         }
 
         if parser.is_escaped {
@@ -390,10 +382,10 @@ pub fn parse_address<'x>(stream: &MessageStream<'x>) -> Address<'x> {
         }
 
         if parser.token_start == 0 {
-            parser.token_start = stream.get_pos();
+            parser.token_start = stream.pos;
             parser.token_end = parser.token_start;
         } else {
-            parser.token_end = stream.get_pos();
+            parser.token_end = stream.pos;
         }
     }
 
@@ -420,7 +412,7 @@ pub fn parse_address<'x>(stream: &MessageStream<'x>) -> Address<'x> {
 mod tests {
     #[test]
     fn parse_addresses() {
-        use crate::parsers::{fields::address::parse_address, message_stream::MessageStream};
+        use crate::parsers::fields::address::parse_address;
 
         use super::*;
 
@@ -941,8 +933,9 @@ mod tests {
         ];
 
         for input in inputs {
-            let mut str = input.0.to_string();
-            let result = parse_address(&MessageStream::new(unsafe { str.as_bytes_mut() }));
+            println!("Testiong {}", input.0);
+            let str = input.0.to_string();
+            let result = parse_address(&mut MessageStream::new(str.as_bytes()));
             let expected: Address = serde_yaml::from_str(input.1).unwrap_or(Address::Empty);
 
             /*if input.0.len() >= 70 {
