@@ -15,12 +15,8 @@ use std::{
 };
 
 use crate::{
-    decoders::{
-        charsets::map::{decoder_default, get_charset_decoder},
-        encoded_word::parse_encoded_word,
-        hex::decode_hex,
-    },
-    parsers::message_stream::MessageStream,
+    decoders::{charsets::map::get_charset_decoder, encoded_word::decode_rfc2047, hex::decode_hex},
+    parsers::message::MessageStream,
     ContentType,
 };
 
@@ -58,24 +54,20 @@ struct ContentTypeParser<'x> {
     is_encoded_attribute: bool,
     is_escaped: bool,
     is_lower_case: bool,
-    is_token_safe: bool,
     is_token_start: bool,
 }
 
 #[inline(always)]
 fn reset_parser(parser: &mut ContentTypeParser) {
     parser.token_start = 0;
-    parser.is_token_safe = true;
     parser.is_token_start = true;
 }
 
 fn add_attribute<'x>(parser: &mut ContentTypeParser<'x>, stream: &MessageStream<'x>) -> bool {
     if parser.token_start > 0 {
-        let mut attr = stream.get_string(
-            parser.token_start - 1,
-            parser.token_end,
-            parser.is_token_safe,
-        );
+        let mut attr = Some(String::from_utf8_lossy(
+            &stream.data[parser.token_start - 1..parser.token_end],
+        ));
 
         if !parser.is_lower_case {
             attr.as_mut().unwrap().to_mut().make_ascii_lowercase();
@@ -98,13 +90,8 @@ fn add_attribute<'x>(parser: &mut ContentTypeParser<'x>, stream: &MessageStream<
 
 fn add_attribute_parameter<'x>(parser: &mut ContentTypeParser<'x>, stream: &MessageStream<'x>) {
     if parser.token_start > 0 {
-        let attr_part = stream
-            .get_string(
-                parser.token_start - 1,
-                parser.token_end,
-                parser.is_token_safe,
-            )
-            .unwrap();
+        let attr_part =
+            String::from_utf8_lossy(&stream.data[parser.token_start - 1..parser.token_end]);
 
         if parser.attr_charset.is_none() {
             parser.attr_charset = attr_part.into();
@@ -134,19 +121,13 @@ fn add_partial_value<'x>(
     if parser.token_start > 0 {
         let in_quote = parser.state == ContentState::AttributeQuotedValue;
 
-        parser.values.push(
-            stream
-                .get_string(
-                    parser.token_start - 1,
-                    if in_quote && to_cur_pos {
-                        stream.get_pos() - 1
-                    } else {
-                        parser.token_end
-                    },
-                    parser.is_token_safe,
-                )
-                .unwrap(),
-        );
+        parser.values.push(String::from_utf8_lossy(
+            &stream.data[parser.token_start - 1..if in_quote && to_cur_pos {
+                stream.pos - 1
+            } else {
+                parser.token_end
+            }],
+        ));
         if !in_quote {
             parser.values.push(" ".into());
         }
@@ -162,11 +143,9 @@ fn add_value<'x>(parser: &mut ContentTypeParser<'x>, stream: &MessageStream<'x>)
 
     let has_values = !parser.values.is_empty();
     let value = if parser.token_start > 0 {
-        stream.get_string(
-            parser.token_start - 1,
-            parser.token_end,
-            parser.is_token_safe,
-        )
+        Some(String::from_utf8_lossy(
+            &stream.data[parser.token_start - 1..parser.token_end],
+        ))
     } else {
         if !has_values {
             return;
@@ -200,16 +179,17 @@ fn add_value<'x>(parser: &mut ContentTypeParser<'x>, stream: &MessageStream<'x>)
 
         if parser.is_encoded_attribute {
             if let (true, decoded_bytes) = decode_hex(value.as_bytes()) {
-                value = get_charset_decoder(
-                    parser
-                        .attr_charset
-                        .as_ref()
-                        .unwrap_or(&"utf-8".into())
-                        .as_bytes(),
-                )
-                .unwrap_or(decoder_default)(&decoded_bytes)
-                .into_owned()
-                .into();
+                value = if let Some(decoder) = parser
+                    .attr_charset
+                    .as_ref()
+                    .and_then(|c| get_charset_decoder(c.as_bytes()))
+                {
+                    decoder(&decoded_bytes).into()
+                } else {
+                    String::from_utf8(decoded_bytes)
+                        .unwrap_or_else(|e| String::from_utf8_lossy(e.as_bytes()).into_owned())
+                        .into()
+                }
             }
             parser.is_encoded_attribute = false;
         }
@@ -239,15 +219,10 @@ fn add_value<'x>(parser: &mut ContentTypeParser<'x>, stream: &MessageStream<'x>)
 
 fn add_attr_position(parser: &mut ContentTypeParser, stream: &MessageStream) -> bool {
     if parser.token_start > 0 {
-        parser.attr_position = stream
-            .get_string(
-                parser.token_start - 1,
-                parser.token_end,
-                parser.is_token_safe,
-            )
-            .unwrap()
-            .parse()
-            .unwrap_or(0);
+        parser.attr_position =
+            String::from_utf8_lossy(&stream.data[parser.token_start - 1..parser.token_end])
+                .parse()
+                .unwrap_or(0);
 
         reset_parser(parser);
         true
@@ -269,7 +244,7 @@ fn merge_continuations(parser: &mut ContentTypeParser) {
     }
 }
 
-pub fn parse_content_type<'x>(stream: &MessageStream<'x>) -> Option<ContentType<'x>> {
+pub fn parse_content_type<'x>(stream: &mut MessageStream<'x>) -> Option<ContentType<'x>> {
     let mut parser = ContentTypeParser {
         state: ContentState::Type,
         state_stack: Vec::new(),
@@ -288,7 +263,6 @@ pub fn parse_content_type<'x>(stream: &MessageStream<'x>) -> Option<ContentType<
         is_continuation: false,
         is_encoded_attribute: false,
         is_lower_case: true,
-        is_token_safe: true,
         is_token_start: true,
         is_escaped: false,
 
@@ -296,7 +270,10 @@ pub fn parse_content_type<'x>(stream: &MessageStream<'x>) -> Option<ContentType<
         token_end: 0,
     };
 
-    while let Some(ch) = stream.next() {
+    let mut iter = stream.data[stream.pos..].iter();
+
+    while let Some(ch) = iter.next() {
+        stream.pos += 1;
         match ch {
             b' ' | b'\t' => {
                 if !parser.is_token_start {
@@ -304,10 +281,10 @@ pub fn parse_content_type<'x>(stream: &MessageStream<'x>) -> Option<ContentType<
                 }
                 if let ContentState::AttributeQuotedValue = parser.state {
                     if parser.token_start == 0 {
-                        parser.token_start = stream.get_pos();
+                        parser.token_start = stream.pos;
                         parser.token_end = parser.token_start;
                     } else {
-                        parser.token_end = stream.get_pos();
+                        parser.token_end = stream.pos;
                     }
                 }
                 continue;
@@ -333,10 +310,11 @@ pub fn parse_content_type<'x>(stream: &MessageStream<'x>) -> Option<ContentType<
                     _ => (),
                 }
 
-                match stream.peek() {
+                match stream.data.get(stream.pos) {
                     Some(b' ' | b'\t') => {
                         parser.state = ContentState::AttributeName;
-                        stream.advance(1);
+                        stream.pos += 1;
+                        iter.next();
 
                         if !parser.is_token_start {
                             parser.is_token_start = true;
@@ -347,6 +325,7 @@ pub fn parse_content_type<'x>(stream: &MessageStream<'x>) -> Option<ContentType<
                         if parser.continuations.is_some() {
                             merge_continuations(&mut parser);
                         }
+
                         return if let Some(content_type) = parser.c_type {
                             Some(ContentType {
                                 c_type: content_type,
@@ -416,9 +395,11 @@ pub fn parse_content_type<'x>(stream: &MessageStream<'x>) -> Option<ContentType<
                 ContentState::AttributeValue | ContentState::AttributeQuotedValue
                     if parser.is_token_start =>
                 {
-                    if let Some(token) = parse_encoded_word(stream) {
+                    if let (d_bytes_read, Some(token)) = decode_rfc2047(stream, stream.pos) {
                         add_partial_value(&mut parser, stream, false);
-                        parser.values.push(token);
+                        parser.values.push(token.into());
+                        stream.pos += d_bytes_read;
+                        iter = stream.data[stream.pos..].iter();
                         continue;
                     }
                 }
@@ -496,12 +477,7 @@ pub fn parse_content_type<'x>(stream: &MessageStream<'x>) -> Option<ContentType<
                 continue;
             }
             b'\r' => continue,
-            0..=0x7f => (),
-            _ => {
-                if parser.is_token_safe {
-                    parser.is_token_safe = false;
-                }
-            }
+            _ => (),
         }
 
         if parser.is_escaped {
@@ -513,20 +489,23 @@ pub fn parse_content_type<'x>(stream: &MessageStream<'x>) -> Option<ContentType<
         }
 
         if parser.token_start == 0 {
-            parser.token_start = stream.get_pos();
+            parser.token_start = stream.pos;
             parser.token_end = parser.token_start;
         } else {
-            parser.token_end = stream.get_pos();
+            parser.token_end = stream.pos;
         }
     }
 
     None
 }
 
+#[cfg(test)]
 mod tests {
+    use crate::parsers::message::MessageStream;
+
     #[test]
     fn parse_content_fields() {
-        use crate::parsers::{fields::content_type::ContentType, message_stream::MessageStream};
+        use crate::parsers::fields::content_type::ContentType;
 
         use super::parse_content_type;
 
@@ -1353,9 +1332,9 @@ mod tests {
         ];
 
         for input in inputs {
-            let mut str = input.0.to_string();
+            let str = input.0.to_string();
 
-            let result = parse_content_type(&MessageStream::new(unsafe { str.as_bytes_mut() }));
+            let result = parse_content_type(&mut MessageStream::new(str.as_bytes()));
             let expected: Option<ContentType> = serde_yaml::from_str(input.1).unwrap_or(None);
 
             /*
