@@ -11,7 +11,7 @@
 
 use std::fmt;
 
-use crate::{parsers::message_stream::MessageStream, DateTime};
+use crate::{parsers::message::MessageStream, DateTime};
 
 impl DateTime {
     /// Returns an ISO-8601 representation of the parsed RFC5322 datetime field
@@ -37,7 +37,7 @@ impl fmt::Display for DateTime {
     }
 }
 
-pub fn parse_date(stream: &MessageStream, abort_on_invalid: bool) -> Option<DateTime> {
+pub fn parse_date(stream: &mut MessageStream) -> Option<DateTime> {
     let mut pos = 0;
     let mut parts = [0u32; 7];
     let mut parts_sizes = [
@@ -57,13 +57,18 @@ pub fn parse_date(stream: &MessageStream, abort_on_invalid: bool) -> Option<Date
     let mut ignore = true;
     let mut comment_count = 0;
 
-    while let Some(ch) = stream.next() {
+    let mut iter = stream.data[stream.pos..].iter();
+
+    while let Some(ch) = iter.next() {
         let mut next_part = false;
+        stream.pos += 1;
 
         match ch {
-            b'\n' => match stream.peek() {
+            b'\n' => match stream.data.get(stream.pos) {
                 Some(b' ' | b'\t') => {
-                    stream.advance(1);
+                    iter.next();
+                    stream.pos += 1;
+
                     if !is_new_token && !ignore && comment_count == 0 {
                         next_part = true;
                     } else {
@@ -78,7 +83,9 @@ pub fn parse_date(stream: &MessageStream, abort_on_invalid: bool) -> Option<Date
                 } else if *ch == b'(' {
                     comment_count += 1;
                 } else if *ch == b'\\' {
-                    stream.skip_byte(&b')');
+                    if let Some(b')') = stream.data.get(stream.pos) {
+                        stream.pos += 1;
+                    }
                 }
                 continue;
             }
@@ -115,11 +122,9 @@ pub fn parse_date(stream: &MessageStream, abort_on_invalid: bool) -> Option<Date
             b'a'..=b'z' | b'A'..=b'Z' => {
                 if pos == 1 {
                     if (1..=2).contains(&month_pos) {
-                        // SAFE: MONTH_HASH's size is u8::MAX
-                        month_hash += unsafe {
-                            *MONTH_HASH
-                                .get_unchecked((if *ch <= b'Z' { *ch + 32 } else { *ch }) as usize)
-                        } as usize;
+                        month_hash += MONTH_HASH
+                            [(if *ch <= b'Z' { *ch + 32 } else { *ch }) as usize]
+                            as usize;
                     }
                     month_pos += 1;
                 }
@@ -133,12 +138,7 @@ pub fn parse_date(stream: &MessageStream, abort_on_invalid: bool) -> Option<Date
                 continue;
             }
             b',' | b'\r' => (),
-            _ => {
-                if abort_on_invalid {
-                    stream.rewind(1);
-                    break;
-                }
-            }
+            _ => (),
         }
 
         if next_part {
@@ -158,8 +158,7 @@ pub fn parse_date(stream: &MessageStream, abort_on_invalid: bool) -> Option<Date
                 parts[2]
             },
             month: if month_pos == 3 && month_hash <= 30 {
-                // SAFE: MONTH_MAP's size is 31
-                (unsafe { *MONTH_MAP.get_unchecked(month_hash) }) as u32
+                MONTH_MAP[month_hash] as u32
             } else {
                 parts[1]
             },
@@ -196,7 +195,7 @@ pub static MONTH_MAP: &[u8; 31] = &[
 
 #[cfg(test)]
 mod tests {
-    use crate::parsers::{fields::date::parse_date, message_stream::MessageStream};
+    use crate::parsers::{fields::date::parse_date, message::MessageStream};
 
     #[test]
     fn parse_dates() {
@@ -261,8 +260,8 @@ mod tests {
         ];
 
         for input in inputs {
-            let mut str = input.0.to_string();
-            match parse_date(&MessageStream::new(unsafe { str.as_bytes_mut() }), false) {
+            let str = input.0.to_string();
+            match parse_date(&mut MessageStream::new(str.as_bytes())) {
                 Some(date) => {
                     //println!("{} -> {}", input.0.escape_debug(), date.to_iso8601());
                     assert_eq!(input.1, date.to_iso8601());
