@@ -9,9 +9,12 @@
  * except according to those terms.
  */
 
-use crate::{parsers::message_stream::MessageStream, MessageHeader, MimeHeader};
+use crate::{MessageHeader, MimeHeader};
 
-use super::fields::{parse_unsupported, MessageField};
+use super::{
+    fields::{parse_unsupported, MessageField},
+    message::MessageStream,
+};
 
 impl<'x> MessageHeader<'x> {
     pub fn new() -> MessageHeader<'x> {
@@ -46,13 +49,16 @@ impl<'x> MimeHeader<'x> {
 }
 
 pub enum HeaderParserResult<'x> {
-    Supported(fn(&mut dyn MessageField<'x>, &MessageStream<'x>)),
+    Supported(fn(&mut dyn MessageField<'x>, &mut MessageStream<'x>)),
     Unsupported(&'x [u8]),
     Lf,
     Eof,
 }
 
-pub fn parse_headers<'x>(header: &mut dyn MessageField<'x>, stream: &MessageStream<'x>) -> bool {
+pub fn parse_headers<'x>(
+    header: &mut dyn MessageField<'x>,
+    stream: &mut MessageStream<'x>,
+) -> bool {
     loop {
         match parse_header_name(stream) {
             HeaderParserResult::Supported(fnc) => fnc(header, stream),
@@ -63,37 +69,30 @@ pub fn parse_headers<'x>(header: &mut dyn MessageField<'x>, stream: &MessageStre
     }
 }
 
-pub fn parse_header_name<'x>(stream: &MessageStream<'x>) -> HeaderParserResult<'x> {
+pub fn parse_header_name<'x>(stream: &mut MessageStream<'x>) -> HeaderParserResult<'x> {
     let mut token_start: usize = 0;
     let mut token_end: usize = 0;
     let mut token_len: usize = 0;
     let mut token_hash: usize = 0;
     let mut last_ch: u8 = 0;
 
-    while let Some(ch) = stream.next() {
+    for ch in stream.data[stream.pos..].iter() {
+        stream.pos += 1;
+
         match ch {
             b':' => {
                 if token_start != 0 {
-                    let field = stream.get_bytes(token_start - 1, token_end).unwrap();
+                    let field = &stream.data[token_start - 1..token_end];
 
                     if (2..=25).contains(&token_len) {
-                        // SAFE: HDR_HASH's size is u8::MAX
-                        token_hash += token_len
-                            + unsafe {
-                                *HDR_HASH.get_unchecked(last_ch.to_ascii_lowercase() as usize)
-                            } as usize;
+                        token_hash +=
+                            token_len + HDR_HASH[last_ch.to_ascii_lowercase() as usize] as usize;
 
                         if (4..=61).contains(&token_hash) {
                             let token_hash = token_hash - 4;
 
-                            // SAFE: HDR_NAME's size is 58
-                               if field.eq_ignore_ascii_case(unsafe {
-                                HDR_NAMES.get_unchecked(token_hash)
-                            }) {
-                                // SAFE: HDR_FNCS's size is 58
-                                return HeaderParserResult::Supported(unsafe {
-                                    *HDR_FNCS.get_unchecked(token_hash)
-                                });
+                            if field.eq_ignore_ascii_case(HDR_NAMES[token_hash]) {
+                                return HeaderParserResult::Supported(HDR_FNCS[token_hash]);
                             }
                         }
                     }
@@ -101,30 +100,28 @@ pub fn parse_header_name<'x>(stream: &MessageStream<'x>) -> HeaderParserResult<'
                 }
             }
             b'\n' => {
-                stream.rewind(1);
+                stream.pos -= 1;
                 return HeaderParserResult::Lf;
             }
             _ => {
                 if !(*ch).is_ascii_whitespace() {
                     if token_start == 0 {
-                        token_start = stream.get_pos();
+                        token_start = stream.pos;
                         token_end = token_start;
                     } else {
-                        token_end = stream.get_pos();
+                        token_end = stream.pos;
                         last_ch = *ch;
                     }
 
                     if let 0 | 9 = token_len {
-                        // SAFE: HDR_HASH's size is u8::MAX
-                        token_hash +=
-                            unsafe { *HDR_HASH.get_unchecked((*ch).to_ascii_lowercase() as usize) }
-                                as usize;
+                        token_hash += HDR_HASH[(*ch).to_ascii_lowercase() as usize] as usize;
                     }
                     token_len += 1;
                 }
             }
         }
     }
+
     HeaderParserResult::Eof
 }
 
@@ -133,7 +130,7 @@ mod tests {
     use crate::parsers::{
         fields::{parse_from, parse_mime_version, parse_received, parse_subject},
         header::parse_header_name,
-        message_stream::MessageStream,
+        message::MessageStream,
     };
 
     use super::HeaderParserResult;
@@ -163,8 +160,8 @@ mod tests {
         ];
 
         for input in inputs {
-            let mut str = input.0.to_string();
-            match parse_header_name(&MessageStream::new(unsafe { str.as_bytes_mut() })) {
+            let str = input.0.to_string();
+            match parse_header_name(&mut MessageStream::new(str.as_bytes())) {
                 HeaderParserResult::Supported(f) => {
                     if let HeaderParserResult::Supported(val) = input.1 {
                         if f as usize == val as usize {
@@ -200,7 +197,7 @@ static HDR_HASH: &[u8] = &[
     62, 62, 62, 62, 62, 62, 62, 62, 62, 62, 62, 62, 62, 62,
 ];
 
-static HDR_FNCS: &[for<'x, 'y> fn(&mut dyn MessageField<'x>, &MessageStream<'x>); 58] = &[
+static HDR_FNCS: &[for<'x, 'y> fn(&mut dyn MessageField<'x>, &mut MessageStream<'x>); 58] = &[
     super::fields::parse_date,
     super::fields::parse_no_op,
     super::fields::parse_sender,
