@@ -13,15 +13,20 @@ use std::borrow::Cow;
 
 use crate::{
     decoders::{
-        base64::Base64Decoder,
-        charsets::map::{decoder_default, get_charset_decoder},
+        base64::decode_base64,
+        charsets::map::get_charset_decoder,
         html::{html_to_text, text_to_html},
-        quoted_printable::QuotedPrintableDecoder,
+        quoted_printable::decode_quoted_printable,
+        DecodeFnc,
     },
     BinaryPart, ContentType, InlinePart, Message, MessageHeader, MessagePart, MimeHeader, TextPart,
 };
 
-use super::{fields::MessageField, header::parse_headers, message_stream::MessageStream};
+use super::{
+    fields::MessageField,
+    header::parse_headers,
+    mime::{get_bytes_to_boundary, seek_next_part, skip_crlf, skip_multipart_end},
+};
 
 #[derive(Debug, PartialEq)]
 enum MimeType {
@@ -38,25 +43,18 @@ enum MimeType {
 }
 
 #[inline(always)]
-fn bytes_to_string<'x>(
-    bytes: &'x [u8],
-    content_type: Option<&ContentType>,
-    is_utf8_safe: bool,
-) -> Cow<'x, str> {
+fn bytes_to_string<'x>(bytes: Cow<'x, [u8]>, content_type: Option<&ContentType>) -> Cow<'x, str> {
     if let Some(content_type) = content_type {
         if let Some(charset) = content_type.get_attribute("charset") {
             if let Some(decoder_fnc) = get_charset_decoder(charset.as_bytes()) {
-                return decoder_fnc(bytes);
+                return decoder_fnc(bytes.as_ref()).into();
             }
         }
     }
 
-    if !is_utf8_safe {
-        decoder_default(bytes)
-    } else {
-        // SAFE: `bytes` slice is ASCII and UTF-8 safe
-        (unsafe { std::str::from_utf8_unchecked(bytes) }).into()
-    }
+    String::from_utf8(bytes.into_owned())
+        .unwrap_or_else(|e| String::from_utf8_lossy(e.as_bytes()).into_owned())
+        .into()
 }
 
 #[inline(always)]
@@ -122,6 +120,17 @@ impl MessageParserState {
     }
 }
 
+pub struct MessageStream<'x> {
+    pub data: &'x [u8],
+    pub pos: usize,
+}
+
+impl<'x> MessageStream<'x> {
+    pub fn new(data: &'x [u8]) -> MessageStream<'x> {
+        MessageStream { data, pos: 0 }
+    }
+}
+
 impl<'x> Message<'x> {
     fn new() -> Message<'x> {
         Message {
@@ -130,17 +139,20 @@ impl<'x> Message<'x> {
         }
     }
 
+    /// Returns `false` if at least one header field was successfully parsed.
+    pub fn is_empty(&self) -> bool {
+        self.header.is_empty()
+    }
+
     /// Parses a byte slice containing the RFC5322 raw message and returns a
     /// `Message` struct.
     ///
     /// This function never panics, a best-effort is made to parse the message and
     /// if no headers are found and empty `Message` struct is returned.
+    /// To make sure parsing was successfull use `is_empty()`.
     ///
-    /// Note: The `parse()` function requires a mutable `[u8]` as input in order
-    /// to perform BASE64/Quoted-Printable decoding in-place.
-    ///
-    pub fn parse(bytes: &'x mut [u8]) -> Message<'x> {
-        let stream = MessageStream::new(bytes);
+    pub fn parse(bytes: &'x [u8]) -> Message<'x> {
+        let mut stream = MessageStream::new(bytes);
 
         let mut message = Message::new();
         let mut message_stack = Vec::new();
@@ -159,7 +171,7 @@ impl<'x> Message<'x> {
             };
 
             // Parse headers
-            if !parse_headers(header, &stream) {
+            if !parse_headers(header, &mut stream) {
                 break;
             }
 
@@ -175,7 +187,7 @@ impl<'x> Message<'x> {
                 {
                     let mime_boundary = ("\n--".to_string() + mime_boundary).into_bytes();
 
-                    if stream.seek_next_part(mime_boundary.as_ref()) {
+                    if seek_next_part(&mut stream, mime_boundary.as_ref()) {
                         let new_state = MessageParserState {
                             in_alternative: state.in_alternative
                                 || mime_type == MimeType::MultipartAlernative,
@@ -190,7 +202,7 @@ impl<'x> Message<'x> {
                         mime_part_header.clear();
                         state_stack.push(state);
                         state = new_state;
-                        stream.skip_crlf();
+                        skip_crlf(&mut stream);
                         continue;
                     } else {
                         mime_type = MimeType::TextOther;
@@ -213,72 +225,76 @@ impl<'x> Message<'x> {
                 state_stack.push(state);
                 message = Message::new();
                 state = new_state;
-                stream.skip_crlf();
+                skip_crlf(&mut stream);
                 continue;
             }
 
-            stream.skip_crlf();
+            skip_crlf(&mut stream);
 
-            let (success, mut is_utf8_safe, mut bytes) = match header
+            let (is_binary, decode_fnc): (bool, DecodeFnc) = match header
                 .get_content_transfer_encoding()
             {
-                Some(encoding) if encoding.eq_ignore_ascii_case("base64") => stream.decode_base64(
-                    state
-                        .mime_boundary
-                        .as_ref()
-                        .map_or_else(|| &[][..], |b| &b[..]),
-                    false,
-                ),
-                Some(encoding) if encoding.eq_ignore_ascii_case("quoted-printable") => stream
-                    .decode_quoted_printable(
+                Some(encoding) if encoding.eq_ignore_ascii_case("base64") => (false, decode_base64),
+                Some(encoding) if encoding.eq_ignore_ascii_case("quoted-printable") => {
+                    (false, decode_quoted_printable)
+                }
+                _ => (true, get_bytes_to_boundary),
+            };
+
+            let (bytes_read, mut bytes) = decode_fnc(
+                &stream,
+                stream.pos,
+                state
+                    .mime_boundary
+                    .as_ref()
+                    .map_or_else(|| &[][..], |b| &b[..]),
+                false,
+            );
+
+            // Attempt to recover contents of an invalid message
+            if bytes_read == 0 {
+                if stream.pos >= stream.data.len() || (is_binary && state.mime_boundary.is_none()) {
+                    break;
+                }
+
+                // Get raw MIME part
+                let (bytes_read, r_bytes) = if !is_binary {
+                    get_bytes_to_boundary(
+                        &stream,
+                        stream.pos,
                         state
                             .mime_boundary
                             .as_ref()
                             .map_or_else(|| &[][..], |b| &b[..]),
                         false,
-                    ),
-                _ => stream.get_bytes_to_boundary(
-                    state
-                        .mime_boundary
-                        .as_ref()
-                        .map_or_else(|| &[][..], |b| &b[..]),
-                ),
-            };
+                    )
+                } else {
+                    (0, None)
+                };
 
-            // Attempt to recover contents of an invalid message
-            if !success {
-                if !stream.is_eof() {
-                    // Get raw MIME part
-                    let (success, r_is_utf8_safe, r_bytes) = stream.get_bytes_to_boundary(
-                        state
-                            .mime_boundary
-                            .as_ref()
-                            .map_or_else(|| &[][..], |b| &b[..]),
-                    );
-                    if !success {
-                        // If there is MIME boundary, ignore it and get raw message
-                        if !stream.is_eof() && state.mime_boundary.is_some() {
-                            let (_, r_is_utf8_safe, r_bytes) =
-                                stream.get_bytes_to_boundary(&[][..]);
-                            if r_bytes.is_some() {
-                                is_utf8_safe = r_is_utf8_safe;
-                                bytes = r_bytes;
-                            } else {
-                                break;
-                            }
+                if bytes_read == 0 {
+                    // If there is MIME boundary, ignore it and get raw message
+                    if state.mime_boundary.is_some() {
+                        let (bytes_read, r_bytes) =
+                            get_bytes_to_boundary(&stream, stream.pos, &[][..], false);
+                        if bytes_read > 0 {
+                            bytes = r_bytes;
+                            stream.pos += bytes_read;
                         } else {
                             break;
                         }
                     } else {
-                        is_utf8_safe = r_is_utf8_safe;
-                        bytes = r_bytes;
+                        break;
                     }
-                    mime_type = MimeType::TextOther;
-                    is_inline = false;
-                    is_text = true;
-                } else if bytes.is_none() {
-                    break;
+                } else {
+                    bytes = r_bytes;
+                    stream.pos += bytes_read;
                 }
+                mime_type = MimeType::TextOther;
+                is_inline = false;
+                is_text = true;
+            } else {
+                stream.pos += bytes_read;
             }
 
             let is_inline = is_inline
@@ -319,9 +335,8 @@ impl<'x> Message<'x> {
             if is_text {
                 let text_part = TextPart {
                     contents: bytes_to_string(
-                        bytes.map_or_else(|| "\n".as_bytes(), |v| v),
+                        bytes.unwrap_or_else(|| vec![b'\n'].into()),
                         header.get_content_type(),
-                        is_utf8_safe,
                     ),
                     header: if !mime_part_header.is_empty() {
                         Some(std::mem::take(&mut mime_part_header))
@@ -358,7 +373,7 @@ impl<'x> Message<'x> {
                     } else {
                         None
                     },
-                    contents: bytes.map_or_else(|| "?".as_bytes(), |v| v).into(),
+                    contents: bytes.unwrap_or_else(|| Cow::from(vec![b'?'])),
                 };
 
                 if add_to_html {
@@ -398,7 +413,7 @@ impl<'x> Message<'x> {
                         }
                     }
 
-                    if stream.skip_multipart_end() {
+                    if skip_multipart_end(&mut stream) {
                         // End of MIME part reached
 
                         if MimeType::MultipartAlernative == state.mime_type
@@ -446,19 +461,19 @@ impl<'x> Message<'x> {
 
                             if let Some(ref mime_boundary) = state.mime_boundary {
                                 // Ancestor has a MIME boundary, seek it.
-                                if stream.seek_next_part(mime_boundary) {
+                                if seek_next_part(&mut stream, mime_boundary) {
                                     continue 'inner;
                                 }
                             }
                         }
                         break 'outer;
                     } else {
-                        stream.skip_crlf();
+                        skip_crlf(&mut stream);
                         // Headers of next part expected next, break inner look.
                         break 'inner;
                     }
                 }
-            } else if stream.is_eof() {
+            } else if stream.pos >= stream.data.len() {
                 break 'outer;
             }
         }
@@ -512,6 +527,8 @@ mod tests {
                     let input = input.split_at_mut(pos);
                     let message = Message::parse(input.0);
 
+                    assert!(!message.is_empty());
+
                     assert_eq!(
                         message,
                         serde_json::from_slice::<Message>(&input.1[SEPARATOR.len()..]).unwrap(),
