@@ -12,4 +12,4 @@
 pub mod fields;
 pub mod header;
 pub mod message;
-pub mod message_stream;
+pub mod mime;
